;+
; NAME:
;	PRC_DUALLIST
;
; PURPOSE:
;	The purpose of this widget/object to provide the user an interactive means of
; 	of adjusting the contents of a list. Items form a 'primary' list may be added to a
;   'secondary' list.  Items in the secondary list may be removed or 'shuffled up' or
;	'shuffled down' relative to each other.  Shuffling can be wrapped in the sense that
;   the last item, if shuffled down, will be wrapped around to be the first item.
;
; CATEGORY:
;	Widgets.
;
; CALLING SEQUENCE:
;	oRef = PRC_DUALLIST()
;
; ARGUMENTS:
;	None
;
; KEYWORDS:  Keywords Flagged by INIT, GET and SET can be accessed through
;	GetProperty and SetProperty methods as well as initialization.  Those flagged with
;	Realize can be set with the REALIZE method.
;
;	NAME (Init/Get/Set),  Object name, a string scalar value (Default = '')
;	UVALUE (Init/Get/Set), Any value for User convenience
;	REALIZE (Init/Set),  Set thios keyword to realize the widget.
;	GROUP (Init/Set,/Realize)  Set this keyword to the group leader of the widget.
;		This must be set each time the widget is realize if a group leader is necessary.
;	NO_BLOCK (Init/Set/Realize) Set this keyword to change the blocking behavior of the widget.
;		(See XManager online help.)
;	TLB (Get) The top level base id for the widget.
;	P_List (Init/Get/Set)  The primary, uneditable list.  Any value is possible,
;		the value (or vector of values must be promotable to string type for
;		display in a list widget.  (Default = ['one, 'Two',...'Ten']  Event though the
;		widget displays P_List as a string, P_List is not converted internally.
;	S_List (Init/Get/Set) The secondary editable list.  Any value is possible,
;		the value (or vector of values must be promotable to string type for
;		display in a list widget. (Default = '')
;	PListID , SListID (Get) The widget ID values for the P_List and the
;		S_List (Underscore removed to make keyword names unique).
;	P_Label (Init/Get/Set) , The primary list label, (Default = 'Primary List')
;	S_Label (Init/Get/Set), The secondary list label (Default = 'Secondary List')
;	Shuffle (Init/Get/Set/Realize) , If set permits user to shuffle values in Secondary List,
;		(Default = 0).  If not set, the shuffle controls are created but not mapped.
;   MUTLIPLE Set this keyword to permit multiple list selection (Default = 0)
;	Title (Init/Get/Set) 	;Title for TLB (Default = 'Please Modify ' + S_label)
;	_Extra (Init/Set)		;extra keywords for List Widgets (except MULTIPLE)
;	Debug (Init/Get/Set)		;convience feature for traceback of errors
;	NOTIFYID (Init/Get/Set)  A 2 column by n row array of widgets to
;		notify when the Add/Remove/MoveUp/MoveDown buttons are pressed.
;		The First column of the array is the Widget ID of the widget to notify
;		The second coloumn is the TOP ID for the notified widget.
;			[WidgetID, TopID]
;		The returned event structure has the following form:
;			Event = {PRC_DUALLIST_BUTTON, ID:NotifyID[0,i],$
;			Top:NotifyID[1,i], $
;			Handler:0L,$				;assigned by XMANAGER
;			ObjRef:SelfMethod.Object, $	;object reference to PRC_DUALLIST
;			Method:SelfMethod.Method,$	; 'string', the method called ADDFIELD, REMOVEFIELD
;													;MOVEUP, MOVEDOWN
;		    P_Select_0: 0L,$	;the zero index of the P_List selection
;			S_Select_0: 0L,$	; the zero index of the S_List selection
;			P_Select_N: 0L,$	; the number of items selected in P_List
;			S_Select_N:0L}	   ;the number of items selected in S_List
;		Set NotifyID equal to a scalar to eliminate this feature.  By default
;		this value is undefined and the widget is Modal.   Be aware of a number of
;		potential pitfalls regarding the last 4 fields of the event.
;		(1) The event sent to NOTIFYID.ID is sent *AFTER* the PRC_DUALLIST_EVENTS
;			event has been completely managed.
;		(2) The P_List selection are static (since no items are ever deleted)... thus
;			even though the event may be regarding the S_LIST, the P_SELECT_0 and (N)
;			fields may retain values from earlier events.   Use the method name to
;			distinguish/discriminate the meaning of the last four fields.
;
;
; METHODS (Public):  See keywords above
;	INIT
;		oRef = PRC_DUALLIST()
;	SETPROPERTY
;		oRef->SetProperty. Keyword = variable,...
;	GETPROPERTY
;		oRef->GetProperty, keyword =variable,...
;	REALIZE
;		oRef->Realize, [/Shuffle], [Group = Group], [No_BLock = N0_Block]
;
; EXAMPLE:
;	To see an example type the following at the command line...
;		oRef = PRC_DUALLIST(/Realize,/Shuffle)
;  	The widget will be realized with dummy values in the Primary list.
;	Using the Add button, add some items from the Primary list to the Secondary list.
;	Shuffle the items in the secondary list around
;	Click done to destroy the widget.
;	Get the contents of the secondary list with oRef->GetProperty, S_list = S_List
;	Print, S_List
;
; MODIFICATION HISTORY
;	Written by Ben Tupper, 2AUG2000.
;	pemaquidriver@tidewater.net
;	248 Lower Round Pond Road
;	POB106
;	Bristol, ME 04539
;
; 7AUG2000 Changed default empty set for S_LIST to an empty string, BT
; 17AUG2000 Added multiple selection with MULTIPLE keyword, BT
; 19AUG2000 Added NOTIFYID capability and adjusted Modality based
;		on contents of NOTIFYID.
; 24AUG2000 Repaired multiple selection shuffles. BT
; 27SEP2000 Added new LOCATETLB
;
;-
;--------
;	LOCATETLB
;-------
PRO PRC_DUALLIST::LOCATETLB
Catch, Error
If Error NE 0 Then Begin
	OK = Error_MEssage(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf

Loc = [0.5,0.5]
Center = 1.0

	; get the device coordinate of the location
Device, Get_Screen_Size=ScreenSize
	;determine the location of the center (or upper left hand corner) of the widget
xLoc = ScreenSize[0]*Loc[0]
yLoc = ScreenSize[1]*Loc[1]

	;retrieve the widget geometry
Geom = Widget_Info(Self.TLB, /Geometry)

	;estimate the actual widget size (it should be close,
	; at least it seems to be on Solaris UNIX and Windows, unless a menu bar is present)
xSize = geom.Scr_XSize  + 2.*geom.margin
ySize = geom.Scr_YSize  + 2.*geom.margin

	;calculate xOffSet, but don't let it run off the display
Xoffset = 0 > (xLoc -  (Cent*Xsize)/2.) < (ScreenSize[0]- xSize)
Yoffset = 0 > (yLoc -  (Cent*Ysize)/2.) < (ScreenSize[1] -ySize)

	; set the offset values
Widget_Control, Self.TLB, XOffset = Xoffset,YOffset =Yoffset

END ;; LocateTLB


;------
;	Events
;------
PRO PRC_DUALLIST_Events, Event

Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack =1)
	Catch, /Cancel
	Return
EndIf

  ;only buttons events are handled
StrName = Tag_Names(Event,/Structure_Name)
If StrName NE 'WIDGET_BUTTON' Then Return

WIdget_Control, EVent.ID, Get_Uvalue = SelfMethod, Get_Value = ThisButton



SelfMethod.Object->GetProperty, NotifyID = NotifyID
	;do notification if neccesary
	; the steps that follow are are modeled on
	; XCOLORS procedure, Fanning Software Consulting, c. 1997-2000
	; www.dfanning.com
Sz = SIze(NotifyID)
If Sz[0] NE 0 Then Begin
	IF Sz[0] EQ 1 THEN Count = 0 ELSE Count = Sz[2]-1
	SelfMethod.Object->GetProperty,PListID = PlistID, SListID = SListID,$
		S_List = S_List ,P_list = P_list
	PListSelect = Widget_Info(PListID,/list_select)
	SListSelect = Widget_Info(SListID,/List_Select)
	FOR i=0,Count Do Begin
		ExternalEvent = {PRC_DUALLIST_BUTTON, ID:NotifyID[0,i],$
			Top:NotifyID[1,i], Handler:0L,$
			ObjRef:SelfMethod.Object, $	;object reference
			Method:SelfMethod.Method,$	;method called
			P_select_0:PListSelect[0], $ ; first P_list selection index
			S_Select_0:SListSelect[0], $ ;first S_List selection index
			P_Select_N:N_elements(PListSelect),$ ;N selected from P_List
			S_Select_N:N_elements(SListSelect)} ;N selected from S_List

		;only sent the event is the id is valid
		If Widget_Info(NotifyID[0,i],/Valid_ID) EQ 1 Then $
			Widget_Control, NotifyID[0,i], Send_Event = ExternalEvent
	EndFor
EndIf


Call_Method,SelfMethod.Method, SelfMethod.Object, Event
END		;Events

;------
;	MoveUp
;------
PRO PRC_DUALLIST::MOVEUP, Event
Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf

  ;get the current S_list selection
S_Select = WIdget_Info(Self.S_ListID, /List_Select)

If S_Select[0] EQ -1 Then Return  ;if nothing selected then return
NList = N_elements(*Self.S_List)
NSelect = N_elements(S_Select)
If S_Select[NSelect-1] GT (NList-1) Then Return ;if the selection is out of bounds the return
If (*Self.S_list)[S_Select[0]] EQ '' Then Return ;if an emtpy string then return

If S_Select[0] EQ 0 Then Begin		; the top most value is easiest
	*Self.S_List = Shift(*Self.S_List, -1)
EndIf Else Begin
    Index = LindGen(NList)
    Index[S_Select[0]-1L:S_Select[NSelect-1]] = Shift(Index[S_Select[0]-1L:S_Select[NSelect-1]],-1)
	 *Self.S_List = (*Self.S_List)[Index]
EndElse

Widget_control, Self.S_ListID, Set_Value = String(*Self.S_List)

END

;------
;	MoveDown
;------
PRO PRC_DUALLIST::MOVEDOWN, Event
Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf

  ;get the current S_list selection
S_Select = WIdget_Info(Self.S_ListID, /List_Select)

If S_Select[0] EQ -1 Then Return  ;if nothing selected then return
NList = N_elements(*Self.S_List)
NSelect = N_elements(S_Select)
If S_Select[NSelect-1L] GT (NList-1L) Then Return ;if the selection is out of bounds the return
If (*Self.S_list)[S_Select[0]] EQ '' Then Return ;if an emtpy string then return

If S_Select[NSELECT-1] EQ (NList-1) Then Begin		; the bottom most value is easiest
	*Self.S_List = Shift(*Self.S_List, 1)
EndIf Else Begin
	Index = LIndGen(NList)
	Index[S_Select[0]:S_Select[NSelect-1]+1L] = Shift(Index[S_Select[0]:S_Select[NSelect-1]+1L],1)
	*Self.S_List = (*Self.S_List)[Index]
	EndElse

Widget_control, Self.S_ListID, Set_Value = STRING(*Self.S_List)

END

;------
;	AddField
;------
PRO PRC_DUALLIST::ADDFIELD, Event

Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf

  ;get the current P_list selection
P_Select = WIdget_Info(Self.P_ListID, /List_Select)

If P_Select[0] NE -1 Then Begin
     ;transfer the field to the S_list
  If N_elements(*Self.S_List) GT 1 Then Begin
     *Self.S_List = [*Self.S_List, (*Self.P_List)[P_Select]]
  EndIf Else Begin
     If (*Self.S_List)[0] EQ '' Then  *Self.S_list = (*Self.P_list)[P_Select] $
       ELse *Self.S_List = [*Self.S_List, (*Self.P_List)[P_Select]]
  EndElse

     ;Update the contents of Value of S_ListID
  Widget_Control, Self.S_ListID, Set_Value = STRING(*Self.S_list)

EndIf	; P_Select NE -1

END  ;AddFIeld

;------
;	RemoveField
;------
PRO PRC_DUALLIST::REMOVEFIELD, Event

Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf

  ;get the current S_list selection
S_Select = WIdget_Info(Self.S_ListID, /List_Select)
OldList = *Self.S_List


If S_Select[0] NE -1 Then Begin
    ;Remove the item
  If N_elements(OldList) GT 1 Then Begin
     Index = LonArr(N_elements(OldList))+1L
	 Index[S_Select] = 0
	 A = WHere(Index GT 0, Count)
	 If Count GT 0 Then *Self.S_List = OldList[A]
	 If Count EQ 0 Then *Self.S_List = ''
     Widget_Control, Self.S_ListID, Set_Value = STRING(*Self.S_List)
  EndIf Else Begin
     *Self.S_list = ''
     Widget_Control, Self.S_ListID, Set_Value = ''	;post a dummy value
  EndElse

EndIf	; S_Select NE -1

ENd	;RemoveField

;------
;	MakeWidget
;------
PRO PRC_DUALLIST::MAKEWIDGET, Group = Group, _Extra = Extra

Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf

  ;if the widget is modal then no events are sent until the widget is killed
Modal = 1
If N_elements(*Self.NotifyID) NE 0 Then Modal = 0
If N_elements(Group) EQ 0 Then Begin
   Self.TLB = Widget_Base(Title = Self.Title, Column = 3, $
   	/Base_Align_Center, Event_Pro = 'PRC_DUALLIST_Events')
EndIf Else Begin
	Self.TLB = Widget_Base(Group = Group,Modal = Modal, $
		Title = Self.Title, Column = 3, $
   		/Base_Align_Center, Event_Pro = 'PRC_DUALLIST_Events')
EndElse

  ;primary side

PrimaryID = WIdget_Base(Self.TLB, /Base_Align_Center,Column = 1,/Frame)
Self.P_LabelID = Widget_Label(PrimaryID, Value = Self.P_label, /Align_Center)

Self.P_ListID = Widget_List(PrimaryID, Value = String(*self.P_List),$
	Ysize = 10 > N_elements(*Self.P_list), _Extra = Extra ,$
		Multiple = Self.Multiple)

	;get the geometry of the primary list to share with the secondary list
Geo = WIdget_info(Self.P_ListID, /Geom)

 ;controls
ControlID = Widget_Base(Self.TLB, /Base_Align_Center, COlumn = 1,$
	Space = 10)
LabelID = WIdget_Label(COntrolID, Value = 'Modify '+Self.S_Label +' Fields',$
	/Align_Center)
TransferID = Widget_Base(ControlID, Column = 1, /Base_Align_Center, /Frame)
	AddID = Widget_Button(TransferID, Value = '>>> Add >>>',$
		Uvalue = {Object:Self, Method:'ADDFIELD'})
	RemoveID = Widget_Button(TransferID, Value = '<< Remove <<',$
		UValue = {Object:Self, Method:'REMOVEFIELD'})

ShuffleID = Widget_Base(ControlID, Column = 1, /Base_Align_Center,/Frame,$
	Map = KeyWord_Set(Self.Shuffle))
	MoveUpID = Widget_Button(ShuffleID, Value = '/\   Move Up   /\', $
		Uvalue = {Object:Self, Method:'MOVEUP'})
	MoveDownID = WIdget_Button(ShuffleID, value = '\/   Move Down   \/',$
		Uvalue = {Object:Self, Method:'MOVEDOWN'})

	DoneID = Widget_Button(ControlID, Value = 'Done', $
		Uvalue = {Object:Self, MEthod:'CONTROL'})

 ;Secondary SIde
SecondaryID = WIdget_Base(Self.TLB, /Base_Align_Center,Column = 1,/Frame)
Self.S_LabelID = Widget_Label(SecondaryID, Value = Self.S_label, /Align_Center)

If N_elements(*Self.S_List) NE  0 Then Begin
  Self.S_ListID = Widget_List(SecondaryID, Value = String(*self.S_List),$
	Ysize = 10 > N_elements(*Self.P_list),Xsize = Geo.Xsize, _Extra = Extra,$
		Multiple = Self.Multiple)
EndIf ELse Begin
  Self.S_ListID = Widget_List(SecondaryID,$
	Ysize = 10 > N_elements(*Self.P_list),Xsize = Geo.Xsize, _Extra = Extra,$
		Multiple = Self.Multiple)
EndElse

LocateTLB,Self.TLB,/Center


END

;------
;	Realize
;------
PRO PRC_DUALLIST::REALIZE, Group = Group, No_Block = No_Block, Shuffle = Shuffle,$
	_Extra = Extra

Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf

If N_elements(Shuffle) NE 0 Then Self.Shuffle = KeyWord_Set(Shuffle)
If XRegistered('PRC_DUALLIST_'+Self.Name) Then Return   ;one exists now

		;otherwise make and realize the widget
Self->MakeWidget, Group = Group
Widget_Control, Self.TLB,/Realize
Xmanager, 'PRC_DUALLIST_'+Self.Name, Self.TLB,$
	No_Block = KeyWord_Set(No_Block) ,  EVent_Handler = 'PRC_DUALLIST_Events'

END

;------
;	Control
;------
PRO PRC_DUALLIST::Control, Event
Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel

	Return
EndIf

Widget_Control, Event.Top, /Destroy

END

;------
;	GetProperty
;------
PRO  PRC_DUALLIST::GetProperty,$
	Name=Name, $
	Uvalue=Uvalue,$
	Debug = Debug,$
	P_Label = P_Label, S_Label= S_Label,$
	P_List = P_List,S_List = S_List,$
	PListID = PListID, SListID = SListID,$
	Shuffle = Shuffle,$
	Multiple = Multiple,$
	Title = Title,$
	NotifyID=NotifyID

Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf
Debug = Self.Debug
Name = Self.Name
If Arg_Present(Uvalue) Then $
	If Ptr_Valid(Self.UValue) Then $
		If N_elements(*Self.UValue) NE 0 Then Uvalue = *Self.Uvalue
TLB = Self.TLB
Shuffle = Self.Shuffle
P_label = Self.P_Label
S_Label = Self.S_label
If Arg_Present(P_List) Then P_List = *Self.P_List
If Arg_Present(S_List) Then S_List = *Self.S_List
If Arg_Present(NotifyID) Then $
	If N_elements(*Self.NotifyID) GT 1 Then NotifyID = *Self.NotifyID

Multiple = Self.Multiple
Title = Self.Title
PListID = Self.P_ListID
SListID = Self.S_ListID

END
;------
;	SetProperty
;------
PRO  PRC_DUALLIST::SetProperty,$
	Realize = Realize,$
	Name=Name, $
	Uvalue=Uvalue,$
	Group=Group, $
	No_Block = No_Block,$
	Debug = Debug,$
	P_Label = P_Label, S_Label= S_Label,$
	Empty_S_List = Empty_S_List, $
	P_List = P_List,S_List = S_List,$
	Shuffle = Shuffle,$
	Multiple = Multiple,$
	Title = Title, $
	NotifyID = NotifyID, $
	_Extra = Extra		;extra keywords for List Widgets

Catch, Error
If Error NE 0 Then Begin
	OK = Error_Message(TraceBack = Self.Debug)
	Catch, /Cancel
	Return
EndIf

If N_elements(Multiple) NE 0 Then Self.Multiple = KeyWord_Set(Multiple)
If N_elements(Debug) NE 0 Then Self.Debug = KeyWord_Set(Debug)
If N_elements(Name) NE 0 Then Self.Name = String(Name[0])
If N_elements(UValue) NE 0 Then Begin
	If N_elements(Uvalue) EQ 1 AND UValue[0] EQ -1 Then $
		UNDEFINE, *Self.Uvalue ELSE *Self.Uvalue = UValue
EndIf

If N_Elements(P_Label) NE 0 Then Self.P_Label = String(P_Label[0])
If N_elements(S_Label) NE 0 Then Self.S_Label = STring(S_Label[0])
If N_elements(Shuffle) NE 0 Then Self.Shuffle = KeyWord_Set(Shuffle)
If N_elements(Title) NE 0 Then Self.Title = String(Title[0])
If N_elements(P_List) NE 0 Then *Self.P_LIst = P_List
If N_Elements(S_List) NE 0 Then *Self.S_List = S_List
If KeyWOrd_Set(Empty_S_List) Then *Self.S_List = ''
If N_elements(NotifyID) NE 0 Then Begin
	If NotifyID[0] EQ -1 Then Self.NotifyID = Ptr_New(/All) Else $
		*Self.NotifyID = NotifyID
EndIf

If KeyWord_Set(Realize) Then Self->Realize, $
	Group = Group, No_Block = No_Block,_Extra = Extra


END
;-------
;	INIT
;------
FUNCTION PRC_DUALLIST::INIT,$
	Name=Name, $				;object name
	Uvalue=Uvalue,$	;User value
	P_Label = P_Label, $	;label Value, Default = 'Primary List'
	S_Label= S_Label,$	;Default = 'Secondary List'
	P_List = P_List,$ ; the primary list
	S_List = S_List,$ ; the secondary list
	Shuffle = Shuffle,$		; if set permits user to shuffle values in Secondary List, Default = 0
	Title = Title, $		;Title for TLB Default = 'Please Modify ' + S_label
	Realize = Realize,$		;realize or not  realize
	_Extra = Extra,$		;extra keywords for List Widgets
	Debug = Debug,$
	Multiple = Multiple,$
	NotifyID=NotifyID, $
	Group = Group, No_Block = No_BLock  ;;for widget realization and control

Catch, Error
If Error NE 0 Then Begin
  OK = Error_Message(TraceBack=Debug)
  Catch,/Cancel
  Return, -1
EndIf

If N_elements(Name) NE 0 Then Self.Name = String(Name[0])
If N_elements(Uvalue) NE 0 Then Self.Uvalue = Ptr_NEW(Uvalue) Else Self.Uvalue = Ptr_NEW(/Allocate)
Self.Shuffle = KeyWord_Set(Shuffle)

If N_elements(P_Label) NE 0 Then Self.P_Label  = String(P_Label[0]) Else $
	Self.P_Label = 'Primary List'
If N_elements(S_Label) NE 0 Then Self.S_Label = String(S_Label[0]) Else $
	Self.S_label = 'Secondary List'

If N_elements(Title) NE 0 Then Title = String(Title[0])  Else Title = 'Please Modify '+Self.S_Label
Self.Title = Title
If N_elements(P_List) NE 0 Then Self.P_List = Ptr_New(String(P_List)) Else $
	Self.P_List = Ptr_new(['One','Two','Three','Four','Five','Six','Seven','Eight','Nine','Ten'])
If N_elements(S_List) NE 0 Then Self.S_List = Ptr_New(String(S_list)) Else $
	Self.S_List = Ptr_New('')
Self.Debug = KeyWord_Set(Debug)
Self.Multiple = KeyWord_Set(Multiple)
If N_elements(NotifyID) NE 0 Then Self.NotifyID = Ptr_NEW(NotifyID) Else $
	Self.NotifyID = Ptr_NEW(/All)

If KeyWord_Set(Realize) Then Self->Realize, $
	Group = Group, No_Block = No_Block, _Extra = Extra

Return,1
END

;-------
;	CleanUp
;-------
PRO PRC_DUALLIST::CleanUp

If Ptr_Valid(Self.Uvalue) Then Ptr_Free, Self.UValue
If Ptr_Valid(Self.P_List) Then Ptr_Free, Self.P_List
If Ptr_Valid(Self.S_List) Then Ptr_Free, Self.S_List
If Ptr_Valid(Self.NotifyID) Then Ptr_Free, Self.NotifyID


END	; CleanUp

;-----
;	DEFINE
;-----

PRO PRC_DUALLIST__Define

Struct = {PRC_DUALLIST,$
	Name:'',$				;object name
	Uvalue:Ptr_NEW(),$	;User value
	TLB: 0L, $		;TopLevel Base for the WIdget
	Title:'',$			;TLB title
	P_LabelID:0L, $	;P_Label WIdgetID
	P_Label:'',$		;label Value
	S_LabelID:0L, $		;S_label widgetID
	S_label:'', $  			;S_label value
	P_ListID:0L,$		;the list widget for primary widget
	S_ListID:0L, $	   ; the list widget for the secondary widget
	P_List: Ptr_NEW(),$ ; the primary list
	S_List:Ptr_NEW() ,$ ; the secondary list
    Shuffle:0,$				;shuffle flag
	Multiple:0,$		;multiple selection flag
	Debug:0, $			;debugging flag
	NotifyID:Ptr_NEW() $		;notifyid array [ID, TOP]
   }
END

FUNCTION PRC_DUALLIST, $
    Name=Name, $				;object name
	Uvalue=Uvalue,$	;User value
	P_Label = P_Label, $	;label Value, Default = 'Primary List'
	S_Label= S_Label,$	;Default = 'Secondary List'
	P_List = P_List,$ ; the primary list
	S_List = S_List,$ ; the secondary list
	Shuffle = Shuffle,$		; if set permits user to shuffle values in Secondary List, Default = 0
	Title = Title, $		;Title for TLB Default = 'Please Modify ' + S_label
	Realize = Realize,$		;realize or not  realize
	_Extra = Extra,$		;extra keywords for List Widgets
	Debug = Debug,$
	Multiple = Multiple,$
	NotifyID = NotifyID,$
	Group = Group, No_Block = No_BLock  ;;for widget realization and control

Return, Obj_New('PRC_DUALLIST', $
	Name=Name, $				;object name
	Uvalue=Uvalue,$	;User value
	P_Label = P_Label, $	;label Value, Default = 'Primary List'
	S_Label= S_Label,$	;Default = 'Secondary List'
	P_List = P_List,$ ; the primary list
	S_List = S_List,$ ; the secondary list
	Shuffle = Shuffle,$		; if set permits user to shuffle values in Secondary List, Default = 0
	Multiple = Multiple,$
	Title = Title, $		;Title for TLB Default = 'Please Modify ' + S_label
	Realize = Realize,$		;realize or not  realize
	_Extra = Extra,$		;extra keywords for List Widgets
	Debug = Debug,$		;convience feature for traceback of errors
	Group = Group, $	; for widget control
	No_Block = No_BLock,$  ;;for widget realization and control
	NotifyID = NotifyID)
END